from urllib.parse import urljoin

import requests
import urllib3
import logging
import webtech

from bs4 import BeautifulSoup as Bs
from bs4 import Comment

from spyderml.lib.file import save_output


def treat_objects(objects: str):
    if "," in objects:
        objects = objects.split(',')
        return objects
    return objects


def update_database():
    webtech.database.update_database(force=True)


def detect_technologies(url):
    webtech.database.update_database()
    # you can use options, same as from the command line
    wt = webtech.WebTech(options={'json': True})

    # scan a single website
    try:
        report = wt.start_from_url(url)
        techs = report['tech']
        print("Site: {} technologies:".format(url))

        for tech in techs:
            print(f"Name: {tech['name']}\tVersion: {tech['version']}")
        print('\n')
    except webtech.utils.ConnectionException:
        print("Connection error")


def spyder_request(target):
    urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
    try:
        r = requests.get(target)
    except requests.exceptions.SSLError:
        print(f"{target}:\nSSL Error")
        exit()
    except requests.exceptions.MissingSchema:
        print(f"{target}:\nInvalid url\n http://?")
        exit()
    except requests.exceptions.InvalidSchema:
        print(f"{target} ERROR")
        exit()
    except requests.exceptions.ConnectionError:
        print(f"Attempt to connect to the host: {target} refused...\n"
              "Tip: Try to exchange https for http or http for https.")
        exit()
    except Exception as e:
        logging.critical(e)
        exit()
    else:
        return r


def soup_tags(document, object, file=None):
    html = Bs(document.content, 'html.parser')
    results = html.find_all(object)
    for result in results:
        if file is not None:
            save_output(filename=file, text=result)
        print(result)


def soup_comments(document, file=None):
    html = Bs(document.content, 'html.parser')
    comments = html.find_all(string=lambda text: isinstance(text, Comment))
    for comment in comments:
        if file is not None:
            save_output(filename=file, text=comment)
        print(comment)


def soup_attrs(document, object, file=None):
    html = Bs(document.content, 'html.parser')
    if type(object) == list:
        for o in object:
            for attribute in html.select(f"[{o}]"):
                if file is not None:
                    save_output(filename=file, text=attribute)
                print(attribute)
    else:
        for attribute in html.select(f"[{object}]"):
            if file is not None:
                save_output(filename=file, text=attribute)
            print(attribute)


def get_js(url, document, file=None):
    html = Bs(document.content, 'html.parser')
    for script in html.find_all("script"):
        if script.attrs.get("src"):
            script_url = urljoin(url, script.attrs.get("src"))
            if file is not None:
                save_output(filename=file, text=script_url)
            print(script_url)



from spyderml.lib.detectos import detect


def open_file(filepath):
    try:
        if '~/' in filepath:
            filepath = filepath.replace('~/',detect())
        with open(filepath, 'r') as file:
            lines = file.read()
            lines = lines.strip()
        lines = lines.split('\n')
        return lines
    except FileNotFoundError:
        print(f"File NotFound({filepath})")
        exit()


def save_output(filename, text):
    if '~' in filename:
        filename = filename.split('~/')
        path = detect()
        filename = path+filename[1]
    with open(filename, 'a+') as file:
        file.write(str(text))
        file.write('\n' )


#!/usr/bin/env python
# -*- coding: utf-8 -*-

import argparse
import sys

from spyderml.lib.asciiarts import banner
from spyderml.lib.utils import spyder_request, treat_objects, soup_tags, soup_comments, soup_attrs, get_js, \
    detect_technologies, update_database
from spyderml.lib.file import open_file

sys.stdin.reconfigure(encoding='utf-8')
sys.stdout.reconfigure(encoding='utf-8')


class SpyderHtml:
    def __init__(self, args):
        self.args = args

    def run(self):
        print(banner())

        if self.args['file']:
            targets = open_file(filepath=self.args['file'])
            htmls = []
            for target in targets:
                if self.args['techs']:
                    detect_technologies(url=target)
                else:
                    htmls.append(spyder_request(target))
            try:
                for html in htmls:
                    if self.args['tags']:
                        tags = treat_objects(objects=self.args['tags'])
                        soup_tags(document=html, object=tags, file=self.args['output'])
                    elif self.args['comments']:
                        soup_comments(document=html, file=self.args['output'])
                    elif self.args['getjs']:
                        get_js(url=self.args['target'], document=html, file=self.args['output'])
                    else:
                        attr = treat_objects(objects=self.args['attribs'])
                        soup_attrs(document=html, object=attr, file=self.args['output'])

            except AttributeError:
                pass

        elif self.args['update']:
            update_database()

        else:
            text = spyder_request(target=self.args['target'])
            if self.args['tags']:
                tags = treat_objects(objects=self.args['tags'])
                soup_tags(document=text, object=tags, file=self.args['output'])
            elif self.args['comments']:
                soup_comments(document=text, file=self.args['output'])
            elif self.args['getjs']:
                get_js(url=self.args['target'], document=text, file=self.args['output'])
            elif self.args['techs']:
                detect_technologies(url=self.args['target'])
            else:
                attr = treat_objects(objects=self.args['attribs'])
                soup_attrs(document=text, object=attr, file=self.args['output'])


def main():
    parser = argparse.ArgumentParser(description="A tool made to facilitate the analysis of html code.")

    group_target = parser.add_mutually_exclusive_group()

    group_action = parser.add_mutually_exclusive_group()

    group_target.add_argument('-t', '--target', type=str, help="Parameter that defines the target URL "
                                                               "http://example.com/index.html")
    group_target.add_argument('-f', '--file', type=str, help="Parameter that defines target URLs")

    group_target.add_argument('--update',help='Flag responsible for updating the database.',
                              default=False, action='store_true')
    group_action.add_argument('--tags', type=str, help="Flag that defines which tags the program will bring")

    group_action.add_argument('--comments', help="Flag that brings the comments", default=False, action='store_true', )

    group_action.add_argument('--attribs', type=str,
                              help="Flag that defines which attributes the application will look for.")

    group_action.add_argument('--getjs', help='Flag that brings all JS files from the page.',
                              default=False, action='store_true')
    group_action.add_argument('--techs', help='Flag trying to discover the technologies of the page.',
                              default=False, action='store_true')
    parser.add_argument('-o', '--output', type=str, help="Flag that defines in which file the command output will be "
                                                        "saved.")

    args = vars(parser.parse_args())

    if not any(args.values()):
        parser.error('No arguments provided.')

    sp = SpyderHtml(args)
    sp.run()


if __name__ == '__main__':
    main()
